/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Vector3f.h"
#include "Vector4f.h"

#include "ClipFlags.h"   /// own header is included last


using namespace hxa7241::graphics;




/// commands ---------------------------------------------------------------------------------------
void ClipFlags::setForPoint
(
	const Vector4f& point
)
{
	xXyYzZ_m = dword( (int(point.getX()                  < 0.0f) << 5) |
	                  (int((point.getW() - point.getX()) < 0.0f) << 4) |
	                  (int(point.getY()                  < 0.0f) << 3) |
	                  (int((point.getW() - point.getY()) < 0.0f) << 2) |
	                  (int(point.getZ()                  < 0.0f) << 1) |
	                  (int((point.getW() - point.getZ()) < 0.0f)) );
}


void ClipFlags::setFromBoundaryCoords
(
	const Vector3f& lower,
	const Vector3f& upper
)
{
	xXyYzZ_m = dword( (int(lower.getX() < 0.0f) << 5) |
	                  (int(upper.getX() < 0.0f) << 4) |
	                  (int(lower.getY() < 0.0f) << 3) |
	                  (int(upper.getY() < 0.0f) << 2) |
	                  (int(lower.getZ() < 0.0f) << 1) |
	                  (int(upper.getZ() < 0.0f)) );
}


void ClipFlags::setFromBoundaryCoords
(
	const float XLoHiYLoHiZLoHi[6]
)
{
	xXyYzZ_m = dword( (int(XLoHiYLoHiZLoHi[0] < 0.0f) << 5) |
	                  (int(XLoHiYLoHiZLoHi[1] < 0.0f) << 4) |
	                  (int(XLoHiYLoHiZLoHi[2] < 0.0f) << 3) |
	                  (int(XLoHiYLoHiZLoHi[3] < 0.0f) << 2) |
	                  (int(XLoHiYLoHiZLoHi[4] < 0.0f) << 1) |
	                  (int(XLoHiYLoHiZLoHi[5] < 0.0f)) );
}




/// constants --------------------------------------------------------------------------------------
const ClipFlags& ClipFlags::NONE()
{
	static const ClipFlags NONE( 0 );
	return NONE;
}


const ClipFlags& ClipFlags::ALL()
{
	static const ClipFlags ALL( 0x3F );
	return ALL;
}
