/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Matrix3f_h
#define Matrix3f_h


#include "Primitives.h"
#include "Vector3f.h"



#include "hxa7241_graphics.h"
namespace hxa7241 { namespace graphics {
	using namespace hxa7241::primitives;


/**
 * a basic 3d matrix of floats, with extra translation capability.<br/><br/>
 *
 * appropriate for general 3d transforms, not perspective projection. more
 * efficient than using a 4d matrix.
 */
class Matrix3f
{
/// standard object services -----------------------------------------------------------------------
public:
	         Matrix3f();
	         Matrix3f( const Vector3f& row0,
	                   const Vector3f& row1,
	                   const Vector3f& row2, const Vector3f& col3 );
	         Matrix3f( float, float, float, float,
	                   float, float, float, float,
	                   float, float, float, float );

	        ~Matrix3f();
	         Matrix3f( const Matrix3f& );
	Matrix3f& operator=( const Matrix3f& );


/// commands ---------------------------------------------------------------------------------------
	        Matrix3f& setRow012Col3( const Vector3f& row0,
	                                 const Vector3f& row1,
	                                 const Vector3f& row2,
	                                 const Vector3f& col3 );
	        void  setElements( float, float, float, float,
	                           float, float, float, float,
	                           float, float, float, float );

	        /**
	         * this = param * this.
	         */
	        Matrix3f& multiplyEq( const Matrix3f& );
	        Matrix3f& scale( const Vector3f& );
	        bool      invert();

	        Matrix3f& setToScale( const Vector3f& );
	        Matrix3f& setToTranslate( const Vector3f& );
	        Matrix3f& setToRotateX( float radians );
	        Matrix3f& setToRotateY( float radians );
	        Matrix3f& setToRotateZ( float radians );
	        Matrix3f& setToRotateXYZ( float radiansX, float radiansY, float radiansZ );


/// queries ----------------------------------------------------------------------------------------
	        const Vector3f& getRow0()                                                         const;
	        const Vector3f& getRow1()                                                         const;
	        const Vector3f& getRow2()                                                         const;
	        const Vector3f& getCol3()                                                         const;
	        void            getRows( Vector3f& row0, Vector3f& row1, Vector3f& row2 )         const;
	        void            getCols( Vector3f& col0, Vector3f& col1, Vector3f& col2 )         const;

	        //void            getRotations( Vector3f& )                                         const;

	        void            multiply( const Vector3f&, Vector3f& )                            const;
	        Vector3f&       multiply( Vector3f& )                                             const;
	        void            multiply( const UnitVector3f&, UnitVector3f& )                    const;
	        UnitVector3f&   multiply( UnitVector3f& )                                         const;

	        bool            operator==( const Matrix3f& )                                     const;
	        bool            operator!=( const Matrix3f& )                                     const;


/// constants --------------------------------------------------------------------------------------
	static const Matrix3f& ZERO();
	static const Matrix3f& IDENTITY();


/// fields -----------------------------------------------------------------------------------------
	Vector3f row0_m;
	Vector3f row1_m;
	Vector3f row2_m;
	Vector3f col3_m;
};


}}




#endif//Matrix3f_h
