/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RenderDataFiling_h
#define RenderDataFiling_h


#include "Primitives.h"
#include "RenderData.h"
#include "RenderingOptions.h"
#include "Scene.h"
#include "RenderingState.h"
#include "ImageCieXyz.h"
#include "RenderingStatus.h"
#include "Thread.h"
#include <vector>
#include <string>
#include <fstream>




#include "perceptuum2_control.h"
namespace perceptuum2 { namespace control {
	using namespace hxa7241::primitives;
	using perceptuum2::modelling::Scene;
	using perceptuum2::imaging::ImageCieXyz;
	using hxa7241::utility::Thread;
	using hxa7241::utility::ThreadFactory;
	using hxa7241::utility::Runnable;
	using std::vector;
	using std::string;
	using std::ifstream;
	using std::ofstream;


class RenderDataFiling
 :	public RenderData, Runnable
{
/// standard object services -----------------------------------------------------------------------
public:
	                  RenderDataFiling( const string& filesPathname, const ThreadFactory& );

	virtual          ~RenderDataFiling();
private:
	                  RenderDataFiling( const RenderDataFiling& );
	RenderDataFiling& operator=( const RenderDataFiling& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void getFilesPathname( string& name ) const;

	static  void getStartFileNames( vector<string>& names );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void save();


/// constants --------------------------------------------------------------------------------------
private:
	static const char SCENE_FILE_EXTENSION[];
	static const char OPTIONS_FILE_EXTENSION[];
	static const char STATE_FILE_EXTENSION[];
	static const char IMAGE_FILE_EXTENSION[];
	static const char STATUS_FILE_EXTENSION[];


/// implementation ---------------------------------------------------------------------------------
	virtual void syncFilesAndObjects( const string& filesPathname );

	static  void syncReadFilesAndObjects( const string& filesPathname, RenderingOptions&, Scene& );
	virtual void syncWriteFilesAndObjects( const string& filesPathname );
	virtual void syncForStartingRender( ifstream& stateFile, ifstream& imageFile );
	virtual void syncForContinuingRender( ifstream& stateFile, ifstream& imageFile );
	virtual void syncStatusFileAndObject( const string& filesPathname );

	virtual void run();
	virtual void saveState();
	virtual void saveImage();
	virtual void saveStatus();


/// fields -----------------------------------------------------------------------------------------
private:
	string filesPathname_m;

	string stateFilePathname_m;
	string imageFilePathname_m;
	string statusFilePathname_m;

	ofstream stateFile_m;
	ofstream imageFile_m;

	RenderingOptions options_m;
	Scene            scene_m;
	RenderingState   state_m;
	ImageCieXyz      image_m;
	RenderingStatus  status_m;

	Thread*                pThread_m;
	const RenderingState*  pStateCopy_m;
	const ImageCieXyz*     pImageCopy_m;
	const RenderingStatus* pStatusCopy_m;

	static const char CLASS_NAME[];
	static const char EXCEPTION_MESSAGE1[];
	static const char EXCEPTION_MESSAGE2[];
	static const char EXCEPTION_MESSAGE3[];
	static const char EXCEPTION_MESSAGE4[];
};


}}




#endif//RenderDataFiling_h
