/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RenderDesktop_h
#define RenderDesktop_h


#include "Primitives.h"
#include "Render.h"
#include "RenderDataFiling.h"
#include "ImageRgba.h"
#include "Thread.h"
#include "ImageConverter.h"
#include <string>
#include <vector>
#include <iosfwd>

#include "hxa7241_utility.h"




#include "perceptuum2_control.h"
namespace perceptuum2 { namespace control {
	using namespace hxa7241::primitives;
	using perceptuum2::imaging::ImageRgba;
	using perceptuum2::imaging::ImageConverter;
	using hxa7241::utility::Runnable;
	using hxa7241::utility::Thread;
	using hxa7241::utility::ThreadFactory;
	using hxa7241::utility::ThreadLock;
	using std::string;
	using std::vector;
	using std::ostream;


class RenderDesktop
 :	public Render, public Runnable
{
/// standard object services -----------------------------------------------------------------------
protected:
	               RenderDesktop( const ThreadFactory&, const string& filesPathname );

	virtual       ~RenderDesktop();
private:
	               RenderDesktop( const RenderDesktop& );
	RenderDesktop& operator=( const RenderDesktop& );


/// commands ---------------------------------------------------------------------------------------
public:
	enum ERgbByteFormat { ARGB, ABGR, RGBA, BGRA };
	virtual void  setRgbImage( dword* pPixels, dword width, dword height, ERgbByteFormat );   /// call immediately after construction
	virtual void  start();
	virtual void  stop();
	virtual void  stopWhenFrameDone();
	virtual void  stopForcibly();
	virtual void  unStop();

	virtual void  setTonemapAdaptationLuminance( float fp );   /// synchronized
	virtual void  persist();                                   /// synchronized


/// queries ----------------------------------------------------------------------------------------
public:
	virtual bool  isThreadRunning()                                 const;
	virtual void  getProjectPathname( string& pathname )            const;   /// synchronized
	static  void  getStartFileNames( vector<string>& names );
	virtual void  getSummaryNames( vector<string>& names )          const;
	virtual void  getSummaries( vector<string>& summaries )         const;   /// synchronized
	virtual void  getStatusLine( string& )                          const;   /// synchronized
	virtual void  getFormattingNames( vector<string>& names )       const;
	virtual void  writeImage( dword format, ostream& imageBytes )   const;   /// synchronized
	virtual float getTonemapAdaptationLuminance()                   const;   /// synchronized
	virtual float getTonemapAveLuminance()                          const;   /// synchronized
	virtual void  getImageSize( dword& width, dword& height )       const;


/// overrides --------------------------------------------------------------------------------------
protected:
	virtual void  run();

	virtual bool  shouldGo()          const;
	virtual bool  shouldStop()        const;
	virtual bool  shouldStopAtFrame() const;

	virtual void  attendToLineDone( dword lineNumber );	      /// synchronized
	virtual void  attendToFrameDone( bool shouldRepaint );    /// synchronized
	virtual void  attendToRenderDone( bool shouldRepaint );   /// synchronized


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  copyInfo();
	virtual void  convertImage();
	virtual void  convertImageLine( dword lineNumber );

	virtual void  setImageConversion( const RenderData&, ImageConverter& );


/// fields -----------------------------------------------------------------------------------------
private:
	RenderDataFiling renderData_m;
	Thread*          pThread_m;
	ThreadLock*      pThreadLock_m;

	enum { PERMISSION_STOP_AT_FRAME = 2 };

	/// synchronized data
	string dataFilePathname_m;
	string optionsSummary_m;
	string sceneSummary_m;
	string statusSummary_m;
	string stateSummary_m;
	string imageSummary_m;
	string statusLine_m;
	float  tonemapAdaptation_m;

	///
	dword  lastPersistTime_m;
	dword  timeBetweenPersists_m;

	///
	ImageRgba imageRgba_m;

	ImageConverter imageConverter_m;

	/// constants
	static const char  CLASS_NAME[];
	static const char* SUMMARY_NAMES[];
	static const char  EXCEPTION_MESSAGE1[];
	static const char  EXCEPTION_MESSAGE2[];
	static const char  EXCEPTION_MESSAGE3[];
};


}}




#endif//RenderDesktop_h
