/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ImageRgba_h
#define ImageRgba_h


#include "Primitives.h"
#include "Sheet.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::utility::Sheet;


class ImageRgba
{
/// standard object services -----------------------------------------------------------------------
public:
	enum ERgbByteFormat { ARGB, ABGR, RGBA, BGRA };

	           ImageRgba();
	           ImageRgba( dword* pPixels, dword width, dword height, ERgbByteFormat );   /// throws
	           ImageRgba( dword width, dword height, ERgbByteFormat );                   /// throws

	          ~ImageRgba();
	           ImageRgba( const ImageRgba& );
	ImageRgba& operator=( const ImageRgba& );


/// commands ---------------------------------------------------------------------------------------
	        void    set( dword* pPixels, dword width, dword height, ERgbByteFormat );   /// throws
	        void    set( dword width, dword height, ERgbByteFormat );                   /// throws

	        void    zeroMemory()                                           const;

	        void    associateAlpha()                                       const;
	        void    associateAlpha( dword row )                            const;
	        void    unassociateAlpha()                                     const;
	        void    unassociateAlpha( dword row )                          const;


/// queries ----------------------------------------------------------------------------------------
	        dword   getWidth()                                             const;
	        dword   getHeight()                                            const;
	        dword   getLength()                                            const;

	        dword&  operator[]( int i )                                    const;
	        dword*  getMemory()                                            const;
	        dword*  getRow( dword y )                                      const;

	        dword   toDword( const dword rgba[4] )                         const;
	        dword   toDword( const float rgba[4] )                         const;
	        dword*  fromDword( dword, dword rgba[4] )                      const;
	        float*  fromDword( dword, float rgba[4] )                      const;

	        dword   removeAlpha( dword )                                   const;
	        dword   associateAlphaP( dword )                               const;
	        dword   unassociateAlphaP( dword )                             const;

	static  bool    isSizeWithinRange( dword width, dword height );


/// fields -----------------------------------------------------------------------------------------
private:
	Sheet<dword> storage_m;

	dword* pPixels_m;
	dword  width_m;
	dword  height_m;

	const int* byteShifts_m;


	static const int  BYTE_SHIFTS_RGBA[4][4];

	static const char CLASS_NAME[];
	static const char EXCEPTION_MESSAGE1[];
};


}}




#endif//ImageRgba_h
