/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "PngWriter.h"
#include "RealPngWriter.h"
#include "Exception.h"

#include "ImageWriter.h"


using namespace perceptuum2::imaging;




/// statics
PngWriter     ImageWriter::pngSrgbWriter_s( true );
PngWriter     ImageWriter::pngWriter_s( false );
RealPngWriter ImageWriter::realPngWriter_s;

const ImageWriter* ImageWriter::writers_s[] = { &pngSrgbWriter_s, &pngWriter_s, &realPngWriter_s };
const dword        ImageWriter::noOfWriters_s = sizeof(writers_s) / sizeof(*writers_s);

const char ImageWriter::CLASS_NAME[]        = "perceptuum2::imaging::ImageWriter";
const char ImageWriter::EXCEPTION_MESSAGE[] = "image file format not available";




/// static queries ---------------------------------------------------------------------------------
void ImageWriter::getFormatNames
(
	vector<string>& names
)
{
	if( names.size() != udword(noOfWriters_s) )
	{
		names.resize( noOfWriters_s );
	}

	for( int i = 0;  i < noOfWriters_s;  ++i )
	{
		writers_s[i]->getName( names[i] );
	}
}


void ImageWriter::write
(
	const ImageCieXyz&    imageXyz,
	const ImageConverter& imageConverter,
	dword                 format,
	ostream&              imageBytes
)
{
	static const char METHOD_NAME[] = "write";

	/// process image through a writer into the stream
	if( (format >= 0) & (format < noOfWriters_s) )
	{
		writers_s[ format ]->write( imageXyz, imageConverter, imageBytes );
	}
	else
	{
		throw hxa7241::utility::Exception( EXCEPTION_MESSAGE, CLASS_NAME, METHOD_NAME );
	}
}
