/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RealPngWriter_h
#define RealPngWriter_h


#include "Primitives.h"
#include "ImageWriter.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using std::string;
	using std::ostream;


/**
 * formats an image into a png of ward realpixels.<br/><br/>
 *
 * resulting png type is: rgb alpha, 8bit depth;<br/>
 * contains the extension chunk: 'xyZe'<br/>
 * the 'xyZe' chunk indicates that the rgb channels contain mantissas of
 * CIE XYZ values, and the alpha channel contains the shared exponent, as
 * described by ward. consequently this is a moderate abuse of the png
 * format.
 *
 * reference:
 * <cite>real pixels, 'graphics gems 2' (AP 91)</cite>
 *
 * @see XyzE
 */
class RealPngWriter
 :	public ImageWriter
{
/// standard object services -----------------------------------------------------------------------
public:
	         RealPngWriter();

	virtual ~RealPngWriter() {}
private:
	         RealPngWriter( const RealPngWriter& );
	RealPngWriter& operator=( const RealPngWriter& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void getName( string& )                                           const;
	virtual void write( const ImageCieXyz&, const ImageConverter&, ostream& ) const;

	static const char* CLASS_NAME();

	static const char EXCEPTION_MESSAGE_1[];
	static const char EXCEPTION_MESSAGE_2[];
	static const char EXCEPTION_MESSAGE_3[];


/// fields -----------------------------------------------------------------------------------------
private:
	static const char FORMAT_NAME[];
	static const char IMAGE_TEXT_COMMENT[];
	static const char IMAGE_TEXT_SOFTWARE[];
	static const char XYZE_CHUNK_NAME[];
};


}}




#endif//RealPngWriter_h
