/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ToneMapperWard94_h
#define ToneMapperWard94_h


#include "Primitives.h"

#include "hxa7241_graphics.h"




#include "perceptuum2_imaging.h"
namespace perceptuum2 { namespace imaging {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;


/**
 * greg ward's tone reproduction operator.<br/><br/>
 *
 * reference:<br/>
 * <cite>'a contrast based scalefactor for luminance display' - ward
 * (graphics gems 4, ap 94)</cite><br/><br/>
 *
 * and a perceptual gamma transform (2.2, which is a reasonable value according
 * to poynton).<br/><br/>
 *
 * input is luminance as a positive fp; output is tone as a positive fp,
 * with the desired range mapping to 0-1.<br/><br/>
 *
 * all luminance in cd/m2
 */
class ToneMapperWard94
{
/// standard object services -----------------------------------------------------------------------
public:
	         ToneMapperWard94();
	         ToneMapperWard94( float minDisplayLuminance,
	                           float maxDisplayLuminance );

	        ~ToneMapperWard94();
	         ToneMapperWard94( const ToneMapperWard94& );
	ToneMapperWard94& operator=( const ToneMapperWard94& );


/// commands ---------------------------------------------------------------------------------------
	       void  set( float minDisplayLuminance,
	                  float maxDisplayLuminance );
	       void  set( float adaptLuminanceForScene );


/// queries ----------------------------------------------------------------------------------------
	       float map  ( float luminance )     const;
	       float unMap( float tone )          const;

	       void  map  ( Vector3f& xyzToTone ) const;
	       void  unMap( Vector3f& toneToXyz ) const;

	       float getAdaptLuminance()          const;


/// implementation ---------------------------------------------------------------------------------
private:
	static float gammarize( float,
	                        float gamma );

	       void  setToneMapping();


/// fields -----------------------------------------------------------------------------------------
private:
	float maxDisplayLuminance_m;
	float adaptLuminanceForScene_m;

	float scaleFactor_m;
	float oneOverScaleFactor_m;


	static const float MIN_LUMINANCE;

	/// defaults
	static const float DEFAULT_MIN_DISPLAY_LUMINANCE;
	static const float DEFAULT_MAX_DISPLAY_LUMINANCE;
	static const float DEFAULT_ADAPT_LUMINANCE_FOR_SCENE;

	static const float GAMMA;
};


}}




#endif//ToneMapperWard94_h
