/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef BssdfConstant_h
#define BssdfConstant_h


#include "Primitives.h"
#include "Bssdf.h"
#include "Light.h"
#include "Float01.h"




#include "perceptuum2_interaction.h"
namespace perceptuum2 { namespace interaction {
	using namespace hxa7241::primitives;
	using hxa7241::utility::Float01;


/**
 * simple perfect-diffuse brdf implementation.<br/><br/>
 *
 * scales by 1 / pi for all directions, distributes uniformly.<br/>
 * makes a good reference and testing implementation.
 */
class BssdfConstant
	: public Bssdf
{
/// standard object services -----------------------------------------------------------------------
public:
	         BssdfConstant( const UnitVector3f& normal,
	                        const UnitVector3f& tangent,
	                        float               refractiveIndexOutside,
	                        float               refractiveIndexInside,
	                        Float01             transparency,
	                        const Light&        reflectivity );

	virtual ~BssdfConstant();
	         BssdfConstant( const BssdfConstant& );
	BssdfConstant& operator=( const BssdfConstant& );


/// commands ---------------------------------------------------------------------------------------
	virtual void  setBssdfConstant( const Light& reflectivity );


/// queries ----------------------------------------------------------------------------------------
	virtual Brdf* clone()                                                                     const;


/// implementation ---------------------------------------------------------------------------------
protected:
	/// overrides
	virtual Float01 getReflectivity_()                                                        const;
	virtual void    getPartitions_( float   proportions[PARTITIONS_MAX],
	                                Float01 peakness[PARTITIONS_MAX],
	                                dword&  length )                                          const;
	virtual Float01 getCos_( dword               part,
	                         const UnitVector3f& outDirection )                               const;
	virtual void    getScaling_( dword               part,
	                             float               IdotN,
	                             float               OdotN,
	                             const UnitVector3f& inDirection,
	                             const UnitVector3f& outDirection,
	                             Light&              scaling )                                const;
	virtual void    getSample_( dword               part,
	                            const Vector2f01&   stochastic,
	                            const UnitVector3f& inDirection,
	                            UnitVector3f&       outDirection,
	                            Light&              scaling )                                 const;


/// fields -----------------------------------------------------------------------------------------
private:
	/// parameters
	Light reflectivitySpectral_m;

	/// auxilliarys
	Float01 reflectivity_m;
	Light   color_m;
};


}}




#endif//BssdfConstant_h
