/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef EmitterPathNode_h
#define EmitterPathNode_h


#include "Primitives.h"
#include "SurfacePoint.h"
#include "LightStream.h"
#include <iosfwd>




#include "perceptuum2_interaction.h"
namespace perceptuum2 { namespace interaction {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::utility::Float01;
	using hxa7241::utility::Float01c;


class EmitterPathNode
	: public SurfacePoint
{
/// standard object services -----------------------------------------------------------------------
public:
	         EmitterPathNode();
	         EmitterPathNode( const Intersection&       intersection,
	                          const ObjectRenderableId& outsideObject,
	                          const Light&              emittance );
	         EmitterPathNode( const Intersection&       intersection,
	                          const ObjectRenderableId& outsideObject,
	                          const LightStream&        inward );

	virtual ~EmitterPathNode();
	         EmitterPathNode( const EmitterPathNode& );
	EmitterPathNode& operator=( const EmitterPathNode& );

	friend  std::ostream& operator<<( std::ostream&, const EmitterPathNode& );


/// commands ---------------------------------------------------------------------------------------
	virtual void  setSurfacePoint( const Intersection&       intersection,
	                               const ObjectRenderableId& outsideObject );
	virtual void  setEmitterPathNode( const Intersection&       intersection,
	                                  const ObjectRenderableId& outsideObject,
	                                  const Light&              emittance );
	virtual void  setEmitterPathNode( const Intersection&       intersection,
	                                  const ObjectRenderableId& outsideObject,
	                                  const LightStream&        inward,
	                                  bool                      isDiffuse );


/// queries ----------------------------------------------------------------------------------------
	virtual void  getNextPathStep( bool          isRoulette,
	                               const Float01 stochastics[7],
	                               Ray&          ray,
	                               Light&        light,
	                               bool&         isDiffuse )                                  const;
	virtual void  getSpreadLight( const Vector3f&     toPosition,
	                              const UnitVector3f& direction,
	                              Float01c            shinynessPoint,
	                              Light&              light )                                 const;

	virtual const LightStream& getInwardLightstream()                                         const;
	virtual bool  isInwardDiffuse()                                                           const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  getSpreadLight( const UnitVector3f& inDirection,
	                              const UnitVector3f& outDirection,
	                              const Light&        inLight,
	                              Float01c            shinynessPoint,
	                              Light&              outLight )                              const;

	virtual void  assign( const SurfacePoint& );
	virtual void  assign( const EmitterPathNode& );


/// fields -----------------------------------------------------------------------------------------
private:
	bool        isRootNode_m;
	Light       rootEmittance_m;
	LightStream inwardLightStream_m;
	bool        isInwardDiffuse_m;
};




std::ostream& operator<<( std::ostream&, const EmitterPathNode& );


}}




#endif//EmitterPathNode_h
