/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "ObjectRenderableId.h"
#include "ObjectRenderable.h"
#include "Vector3f.h"
#include "Light.h"
#include "Float01.h"
#include "hxa7241math.h"

#include "Medium.h"   /// own header is included last


using namespace perceptuum2::interaction;




/// standard object services -----------------------------------------------------------------------
Medium::Medium
(
	const ObjectRenderableId& object
)
 :	object_m( object )
{
}


Medium::~Medium()
{
}


Medium::Medium( const Medium& other )
 :	object_m( other.object_m )
{
	//Medium::operator=( other );
}


Medium& Medium::operator=( const Medium& other )
{
	//if( &other != this )
	{
		object_m = other.object_m;
	}

	return *this;
}




/// queries ----------------------------------------------------------------------------------------
bool Medium::getParticipationRoulette
(
	const Float01   stochastic,
	const Vector3f& pos1,
	const Vector3f& pos2,
	Light&          //light
) const
{
	/// uniform colorless medium only
	/// outscattering and absorbtion only (ignoring inscattering)
	/// light out = (e ^ (-extinction coefficient * distance)) * light in

	const float extinctionCoefficient = object_m->getExtinctionCoefficient( object_m, pos1 );
	const float scaling               = float( ::exp( -extinctionCoefficient * pos1.distance( pos2 ) ) );

	return stochastic < scaling;
}


void Medium::getParticipationScaling
(
	const Vector3f& pos1,
	const Vector3f& pos2,
	Light&          scaling
) const
{
	scaling = Light::ONE();
	participateLight( pos1, pos2, scaling );
}


void Medium::participateLight
(
	const Vector3f& pos1,
	const Vector3f& pos2,
	Light&          light
) const
{
	/// uniform colorless medium only
	/// outscattering and absorbtion only (ignoring inscattering)
	/// light out = (e ^ (-extinction coefficient * distance)) * light in

	const float extinctionCoefficient = object_m->getExtinctionCoefficient( object_m, pos1 );
	const float scaling               = float( ::exp( -extinctionCoefficient * pos1.distance( pos2 ) ) );

	light *= scaling;
}
