/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Intersection_h
#define Intersection_h


#include "Primitives.h"
#include "Vector3f.h"
#include "UnitVector3f.h"
#include "Vector2.h"
#include "ObjectRenderableId.h"
#include <iosfwd>

#include "hxa7241_graphics.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::graphics::Vector2f;
	using hxa7241::graphics::Transform;
	using perceptuum2::modelling::ObjectRenderableId;


class Intersection
{
/// standard object services -----------------------------------------------------------------------
public:
	         Intersection();

	        ~Intersection();
	         Intersection( const Intersection& );
	Intersection& operator=( const Intersection& );

	friend  std::ostream& operator<<( std::ostream&, const Intersection& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  set( const Vector3f& position, const UnitVector3f& normal, const Vector2f& coords,
	                   const ObjectRenderableId& object );

	        Intersection& transformEq( const Transform& );


/// queries ----------------------------------------------------------------------------------------
public:
//	        const Vector3f&           getPosition() const;
//	        const UnitVector3f&       getNormal()   const;
//	        const Vector2f&           getCoords()   const;
//	        const ObjectRenderableId& getObject()   const;


/// fields -----------------------------------------------------------------------------------------
public:
	Vector3f           position_m;
	UnitVector3f       normal_m;   /// structural normal - not interpolated or bump mapped
	Vector2f           coords_m;
	ObjectRenderableId object_m;
};








std::ostream& operator<<( std::ostream&, const Intersection& );




/// inlines ///

/// queries ----------------------------------------------------------------------------------------
/*inline
const Vector3f& Intersection::getPosition() const
{
	return position_m;
}


inline
const UnitVector3f& Intersection::getNormal() const
{
	return normal_m;
}


inline
const Vector2f& Intersection::getCoords() const
{
	return coords_m;
}


inline
const ObjectRenderableId& Intersection::getObject() const
{
	return object_m;
}*/


}}




#endif//Intersection_h
