/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ObjectInstanceMoving_h
#define ObjectInstanceMoving_h


#include "Primitives.h"
#include "ObjectInstance.h"
#include "Vector3f.h"
#include "UnitVector3f.h"
#include "Quaternion.h"
#include <deque>

#include "hxa7241_utility.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::graphics::Quaternion;
	using hxa7241::graphics::Transplace;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::deque;


/**
 * a group of moving renderable instances.<br/><br/>
 *
 * this class is temporal, its base is non-temporal.
 * the overriden update method uses time dependent info in this object to
 * set the base object for a particular timepoint.
 */
class ObjectInstanceMoving
	: public ObjectInstance
{
/// standard object services -----------------------------------------------------------------------
public:
	         ObjectInstanceMoving( const Scene*, SerializerIn&, bool );
protected:
	         ObjectInstanceMoving( const Scene* );
public:
	virtual ~ObjectInstanceMoving();
	         ObjectInstanceMoving( const ObjectInstanceMoving& );
	ObjectInstanceMoving& operator=( const ObjectInstanceMoving& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual bool  update( float time );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual const char* getClassName()                                                        const;


/// serialization ----------------------------------------------------------------------------------
public:
	friend  SerializerOut& operator<<( SerializerOut&, const ObjectInstanceMoving& );
	friend  SerializerIn&  operator>>( SerializerIn&,        ObjectInstanceMoving& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const ObjectRenderable& );
	virtual void  assign( const ObjectInstance& );
	virtual void  assign( const ObjectInstanceMoving& );

	class Motion
	{
	/// standard object services ---------------------------------------------------
	public:
		 Motion() {}
		~Motion() {}
		 Motion( const Motion& ) {}
		 Motion& operator=( const Motion& ) { return *this; }

	/// queries --------------------------------------------------------------------
		dword getObjectIndex()                                                                const;
		bool  setTransform( float time, float previousTime, bool isTimeChecked, Transplace& ) const;

	/// serialization --------------------------------------------------------------
		friend  SerializerOut& output( SerializerOut&, const Motion&, const Scene& );
		friend  SerializerIn&  input ( SerializerIn&,        Motion&, const Scene& );
		static const char* CLASS_NAME();
		static const char* CLASS_DESCRIPTION();

	/// fields ---------------------------------------------------------------------
	private:
		dword      objectIndex_m;

		float      timeStart_m;
		float      timeEnd_m;

		Quaternion   rotationStart_m;
		float        motionAngle_m;
		UnitVector3f motionAxis_m;
		Vector3f     positionStart_m;
		Vector3f     positionEnd_m;
	};
	friend  SerializerOut& output( SerializerOut&, const Motion&, const Scene& );
	friend  SerializerIn&  input ( SerializerIn&,        Motion&, const Scene& );

	virtual void  setBase();


/// fields -----------------------------------------------------------------------------------------
private:
	deque<Motion> motions_m;

	static const char DESERIALIZATION_WARNING_1[];
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const ObjectInstanceMoving& );
SerializerIn&  operator>>( SerializerIn&,        ObjectInstanceMoving& );


SerializerOut& output( SerializerOut&, const ObjectInstanceMoving::Motion&, const Scene& );
SerializerIn&  input ( SerializerIn&,        ObjectInstanceMoving::Motion&, const Scene& );


}}




#endif//ObjectInstanceMoving_h
