/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ObjectMeshCylinder_h
#define ObjectMeshCylinder_h


#include "Primitives.h"
#include "ObjectMesh.h"
#include "TriangleImplicit.h"
#include "Vector3f.h"
#include <vector>




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::vector;


/**
 * a cylinder, made of a triangle mesh.
 */
class ObjectMeshCylinder
	: public ObjectMesh
{
/// standard object services -----------------------------------------------------------------------
public:
	         ObjectMeshCylinder( const Scene*, SerializerIn&, bool );
protected:
	         ObjectMeshCylinder( const Scene* );
public:
	virtual ~ObjectMeshCylinder();
	         ObjectMeshCylinder( const ObjectMeshCylinder& );
	ObjectMeshCylinder& operator=( const ObjectMeshCylinder& );


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const ObjectMeshCylinder& );
	friend  SerializerIn&  operator>>( SerializerIn&,        ObjectMeshCylinder& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const ObjectRenderable& );
	virtual void  assign( const ObjectMesh& );
	virtual void  assign( const ObjectMeshCylinder& );

	static  void  makeCylinder( float radius, float height, bool isCurved, dword resolution,
	                            vector<Vector3f>& vertexs, vector<TriangleImplicit>& faces );


/// fields -----------------------------------------------------------------------------------------
private:
	float radius_m;
	float height_m;
	float isCurved_m;
	dword resolution_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const ObjectMeshCylinder& );
SerializerIn&  operator>>( SerializerIn&,        ObjectMeshCylinder& );


}}




#endif//ObjectMeshCylinder_h
