/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ObjectRenderable_h
#define ObjectRenderable_h


#include "Primitives.h"
#include "Bound.h"
#include "CieXyz.h"
#include "Float01.h"
#include <vector>

#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::CieXyz;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::graphics::Transclip;
	using hxa7241::graphics::ClipFlags;
	using hxa7241::utility::Float01;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::vector;


class ObjectRenderable
{
/// standard object services -----------------------------------------------------------------------
protected:
	         ObjectRenderable( const Scene* );
public:
	virtual ~ObjectRenderable();
protected:
	         ObjectRenderable( const ObjectRenderable& );
	ObjectRenderable& operator=( const ObjectRenderable& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual bool  update( float time )                                                           =0;


/// queries ----------------------------------------------------------------------------------------
public:
	virtual const CieXyz& getEmittanceSum()                                                const;
	virtual Float01 getSpecularness()                                                      const;
	virtual dword   getHierarchyLevel()                                                    const;
	virtual dword   getSubPartCount()                                                      const;

	virtual const Bound& getBound()                                                        const;


	virtual bool    getEmitted( const Vector3f& posRands,
	                            vector<float>&  compositionStorage,
	                            Intersection&,
	                            Qualities& )                                               const =0;

	virtual bool    getIntersection( const ObjectRenderableId&,
	                                 const Ray&,
	                                 float& distance,
	                                 Intersection& )                                       const =0;
	virtual void    getObjectsForProjection( const Transclip& transform,
	                                         ClipFlags        clipFlags,
	                                         vector<ProjectableObjectRef>& )               const =0;
	virtual void    getMesh( Mesh&, const Mesh** )                                         const =0;

	virtual void    getQualities( const Intersection&, Qualities& )                        const =0;

	virtual Float01 getTransparency( const ObjectRenderableId&,
	                                 const Vector3f& position )                            const =0;
	virtual float   getExtinctionCoefficient( const ObjectRenderableId&,
	                                          const Vector3f& position )                   const;
	virtual float   getRefractiveIndex( const ObjectRenderableId&,
	                                    const Vector3f& position )                         const =0;
	virtual const CieXyz& getEmittance( const ObjectRenderableId& )                        const =0;

	virtual const char* getClassName()                                                     const =0;


/// serialization ----------------------------------------------------------------------------------
public:
	friend  SerializerOut& operator<<( SerializerOut&, const ObjectRenderable& );
	friend  SerializerIn&  operator>>( SerializerIn&,        ObjectRenderable& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void   setEmittanceSum( const CieXyz& );
	virtual void   setSpecularness( Float01 );
	virtual void   setHierarchyLevel( dword );
	virtual void   setSubPartCount( dword );

	virtual Bound& bound();

	virtual const Scene* getScene() const;

	virtual void  assign( const ObjectRenderable& );


/// fields -----------------------------------------------------------------------------------------
private:
	Bound   bound_m;

	CieXyz  emittance_m;
	Float01 specularness_m;
	dword   hierarchyLevel_m;
	dword   subPartCount_m;

	const Scene* pScene_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const ObjectRenderable& );
SerializerIn&  operator>>( SerializerIn&,        ObjectRenderable& );


}}




#endif//ObjectRenderable_h
