/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ObjectRenderableId_h
#define ObjectRenderableId_h


#include "Primitives.h"
#include <iosfwd>




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;


class ObjectRenderableId
{
/// standard object services -----------------------------------------------------------------------
public:
	         ObjectRenderableId();
	explicit ObjectRenderableId( const ObjectRenderable* pObject, dword primitive =0 );
	         ObjectRenderableId( const ObjectRenderableId&, dword primitive );

	        ~ObjectRenderableId();
	         ObjectRenderableId( const ObjectRenderableId& );
	ObjectRenderableId& operator=( const ObjectRenderableId& );

	friend  std::ostream& operator<<( std::ostream&, const ObjectRenderableId& );


/// commands ---------------------------------------------------------------------------------------
public:
	        bool  push( byte subPartId );
	        bool  push( const ObjectRenderable*, byte subPartId );
	        bool  pop ( const ObjectRenderable* );
	        bool  shift( const ObjectRenderable* );


/// queries ----------------------------------------------------------------------------------------
public:
	        const ObjectRenderable* operator->()                  const;

	        const ObjectRenderable* getMainObject()               const;
	        dword getPrimitiveId()                                const;
	        byte  getTop()                                        const;
	        byte  getBottom()                                     const;

	        bool  isValid()                                       const;
	        bool  operator==( const ObjectRenderableId& )         const;
	        bool  operator!=( const ObjectRenderableId& )         const;
	        bool  isMainObjectsEqual( const ObjectRenderableId& ) const;

	static  dword getMaxStackSize();
	static  dword getMaxStackWidth();


/// implementation ---------------------------------------------------------------------------------
private:
	static  void zeroStack( byte[] );
	static  void copyStack( const byte[], byte[] );
	static  bool isEqualStack( const byte[], const byte[], dword size );


/// fields -----------------------------------------------------------------------------------------
private:
	#define STACK_2_DWORDS true
	static const dword STACK_SIZE_MAX  = 8;
	static const dword STACK_WIDTH_MAX = 1 << BYTE_BITS;

	const ObjectRenderable* pObject_m;
	byte  stack_m[STACK_SIZE_MAX];
	dword stackEnd_m;

	dword primitive_m;
};








std::ostream& operator<<( std::ostream&, const ObjectRenderableId& );




/// INLINES ///


/// queries ----------------------------------------------------------------------------------------
inline
const ObjectRenderable* ObjectRenderableId::operator->() const
{
	return pObject_m;
}


inline
const ObjectRenderable* ObjectRenderableId::getMainObject() const
{
	return pObject_m;
}


inline
dword ObjectRenderableId::getPrimitiveId() const
{
	return primitive_m;
}


inline
bool ObjectRenderableId::isValid() const
{
	return pObject_m != 0;
}




}}




#endif//ObjectRenderableId_h
