/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Qualities_h
#define Qualities_h


#include "Primitives.h"
#include "UnitVector3f.h"
#include "CieXyz.h"
#include "Float01.h"

#include "hxa7241_utility.h"
#include "hxa7241_graphics.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::graphics::CieXyz;
	using hxa7241::graphics::Transform;
	using hxa7241::utility::Float01;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;


class Qualities
{
/// standard object services -----------------------------------------------------------------------
public:
	         Qualities();

	        ~Qualities();
	         Qualities( const Qualities& );
	Qualities& operator=( const Qualities& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void setToDefault();

	        Qualities& transformEq( const Transform& );


/// queries ----------------------------------------------------------------------------------------
public:
	        Float01 getSpecularness() const;
	static  Float01 getSpecularness( Float01 roughnessX, Float01 roughnessY,
	                                 const CieXyz& reflectivitySpecular );


/// serialization ----------------------------------------------------------------------------------
public:
	friend  SerializerOut& operator<<( SerializerOut&, const Qualities& );
	friend  SerializerIn&  operator>>( SerializerIn&,        Qualities& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
private:
	static  void  copyString( const char*, dword length, char* );


/// fields -----------------------------------------------------------------------------------------
public:
	UnitVector3f normal_m;   /// after interpolation or bump map...
	UnitVector3f tangent_m;
	CieXyz       reflectivityDiffuse_m;
	CieXyz       reflectivitySpecular_m;
	Float01      roughnessX_m;
	Float01      roughnessY_m;
	Float01      transparency_m;
	float        refractiveIndex_m;
	CieXyz       emitted_m;
	Float01      emittedRoughness_m;
	char         brdfClassName_m[17];
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const Qualities& );
SerializerIn&  operator>>( SerializerIn&,        Qualities& );


}}




#endif//Qualities_h
