/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Scene_h
#define Scene_h


#include "Primitives.h"
#include "ObjectDefault.h"
#include "Material.h"
#include "ObjectInstantiator.h"
#include <string>
#include <deque>
#include <map>

#include "hxa7241_utility.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::string;
	using std::deque;
	using std::map;


class Scene
{
/// standard object services -----------------------------------------------------------------------
public:
	                Scene();

	virtual        ~Scene();
private:
	                Scene( const Scene& );
	         Scene& operator=( const Scene& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  setLightsourceEmittanceThreshold( float );

	/// for reading
	virtual void  addNameOfMaterial( const string& );
	virtual void  addNameOfObject( const string& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual float getLightsourceThreshold()                                                   const;

	virtual void  getSummary( string& )                                                       const;
	virtual float getTimeBegin()                                                              const;
	virtual float getTimeEnd()                                                                const;

	virtual bool  update( float time )                                                        const;

	virtual ObjectRenderableId getRootObject()                                                const;
	virtual ObjectRenderableId getDefaultObject()                                             const;
	virtual const ObjectRenderable* getObject( dword index )                                  const;
	virtual ObjectRenderable*       getObjectNonConst( dword index )                          const;
	virtual const Material&         getMaterial( dword index )                                const;

	/// for reading
	virtual dword  getMaterialIndex( const string& )                                          const;
	virtual string getMaterialName( dword index )                                             const;
	virtual dword  getObjectIndex( const string& )                                            const;
	virtual string getObjectName( dword index )                                               const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const Scene& );
	friend  SerializerIn&  operator>>( SerializerIn&,        Scene& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
private:
	static  void   addName ( const string&       name,
	                         const char          type[],
	                         map<string, dword>& mapping );
	static  dword  getIndex( const string&             name,
	                         const char                type[],
	                         const map<string, dword>& mapping );
	static  string getName ( dword                     index,
	                         const map<string, dword>& mapping );
	static  void   throwNameException( const string& name,
	                                   const char    type[],
	                                   const char    problem[],
	                                   const char    method[] );

	virtual void  enforceConstraints();


/// fields -----------------------------------------------------------------------------------------
private:
	float lightsourceThreshold_m;

	map<string, dword> materialNames_m;
	map<string, dword> objectNames_m;

	deque<ObjectInstantiator> objects_m;
	ObjectDefault             defaultObject_m;

	deque<Material> materials_m;
	Material        defaultMaterial_m;

	float timeBegin_m;
	float timeEnd_m;

	static const char START_TIME_LABEL[];
	static const char END_TIME_LABEL[];
	static const char OBJECT_COUNT_LABEL[];
	static const char MATERIAL_COUNT_LABEL[];
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const Scene& );
SerializerIn&  operator>>( SerializerIn&,        Scene& );


}}




#endif//Scene_h
