/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Illumination_h
#define Illumination_h


#include "Primitives.h"
#include "EmitterPath.h"
#include "PhotonMap.h"
#include "LightStream.h"
#include <vector>

#include "perceptuum2_interaction.h"
#include "hxa7241_graphics.h"
#include "hxa7241_utility.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using perceptuum2::interaction::EyePathNode;
	using perceptuum2::interaction::Light;
	using perceptuum2::interaction::LightStream;
	using hxa7241::graphics::UnitVector3f;
	using hxa7241::utility::Float01;
	using hxa7241::utility::Float01c;
	using std::vector;


class Illumination
{
/// standard object services -----------------------------------------------------------------------
public:
	         Illumination( dword    emitterPathNumber,
	                       dword    photonMapGeneralSize,
	                       //dword    photonMapCausticSize,
	                       //dword    gatherRayCount,
	                       float    lightsourceEmittanceThreshold,
	                       dword    pathLengthLimit,
	                       Float01c pathEndThreshold,
	                       bool     isLogging );

	virtual ~Illumination();
private:
	         Illumination( const Illumination& );
	Illumination& operator=( const Illumination& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  makeIllumination( const SpacetimeIndex&,
	                                SamplingControl& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual void  getIllumination( const SamplingControlDiffuse& samplingControl,
	                               const SpacetimeIndex&         sceneIndex,
	                               const EyePathNode&            lastEyePathNode,
	                               const MediaStack&             lastEyePathNodeMedia,
	                               const UnitVector3f&           eyeDirection,
	                               Float01c                      shinynessPoint,
	                               Light&                        illumination )               const;
	virtual void  getDirectIllumination( const SamplingControlDiffuse& samplingControl,
	                                     const SpacetimeIndex&         sceneIndex,
	                                     const EyePathNode&            lastEyePathNode,
	                                     const MediaStack&             lastEyePathNodeMedia,
	                                     const UnitVector3f&           eyeDirection,
	                                     Float01c                      shinynessPoint,
	                                     Light&                        illumination )         const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  makeDirectIllumination( const SpacetimeIndex&,
	                                      SamplingControl& );
	virtual void  makePhotonMapIllumination( const SpacetimeIndex&,
	                                         bool isDiscludeDirectLighting,
	                                         SamplingControl& );

	static  void  getPhotonMapIllumination( const PhotonMap&    photonMap,
	                                        const EyePathNode&  lastEyePathNode,
	                                        const UnitVector3f& eyeDirection,
	                                        const Float01c      shinynessPoint,
	                                        Light&              eyewardLight );
//	virtual void  gather( const SamplingControlDiffuse& samplingControl,
//	                      const SpacetimeIndex&         sceneIndex,
//	                      const EyePathNode&            localNode,
//	                      const MediaStack&             localNodeMedia,
//	                      const UnitVector3f&           eyeDirection,
//	                      Float01c                      shinynessPoint,
//	                      Light&                        light )                               const;


/// fields -----------------------------------------------------------------------------------------
private:
	/// illumination containers
	vector<EmitterPath> emitterPaths_m;
	PhotonMapBuffers    photonMapBuffers_m;
	PhotonMap           photonMap_m;
	PhotonMap           photonMapDirect_m;
	PhotonMap           photonMapCaustic_m;

	/// calculation buffers
	EmitterPath                 photonPathBuffer_m;
	vector<float>               lightsourceHistogramStorage_m;
	mutable vector<LightStream> spreadStorage_m;
//	mutable vector<float>       importanceStorage_m;
//	mutable vector<float>       gridStorage_m;

//	dword gatherRayCount_m;

	/// overall statistics
	dword totalGeneralPathCount_m;
	dword totalGeneralNodeCount_m;
	dword totalCausticPathCount_m;
	dword totalCausticNodeCount_m;
	mutable dword totalIlluminationQueriesCount_m;

	bool isLogging_m;
};


}}




#endif//Illumination_h
