/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef SamplingControlDiffuse_h
#define SamplingControlDiffuse_h


#include "Primitives.h"
#include "SobolSequence.h"
#include <vector>

#include "hxa7241_utility.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using hxa7241::utility::SobolSequence;
	using hxa7241::utility::Float01;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::vector;


/**
 * diffuse gather sampling controller.<br/><br/>
 *
 * provides sampling coordinates for diffuse gathering.<br/>
 */
class SamplingControlDiffuse
{
public:
	static const dword COORDS_COUNT = 3;

/// standard object services -----------------------------------------------------------------------
public:
	         /** @param sobolDimensions must have length == COORDS_COUNT */
	         SamplingControlDiffuse( const dword sobolDimensions[COORDS_COUNT] );

	        ~SamplingControlDiffuse();
	         SamplingControlDiffuse( const SamplingControlDiffuse& );
	SamplingControlDiffuse& operator=( const SamplingControlDiffuse& );


/// commands ---------------------------------------------------------------------------------------
	        void  advance();


/// queries ----------------------------------------------------------------------------------------
	        void  getSampleCoords( dword   index,
	                               Float01 coords[COORDS_COUNT] )                             const;


/// serialization ----------------------------------------------------------------------------------
	friend  SerializerOut& operator<<( SerializerOut&, const SamplingControlDiffuse& );
	friend  SerializerIn&  operator>>( SerializerIn&,        SamplingControlDiffuse& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// fields -----------------------------------------------------------------------------------------
private:
	mutable SobolSequence sobols_m[COORDS_COUNT];
	mutable vector<float> coordCache_m;
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const SamplingControlDiffuse& );
SerializerIn&  operator>>( SerializerIn&,        SamplingControlDiffuse& );


}}




#endif//SamplingControlDiffuse_h
