/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef SpacetimeIndex_h
#define SpacetimeIndex_h


#include "Primitives.h"
#include "RenderingTime.h"

#include "perceptuum2_modelling.h"
#include "hxa7241_graphics.h"




#include "perceptuum2_transport.h"
namespace perceptuum2 { namespace transport {
	using namespace hxa7241::primitives;
	using perceptuum2::modelling::Scene;
	using perceptuum2::modelling::ObjectRenderableId;
	using perceptuum2::modelling::Ray;
	using perceptuum2::modelling::Intersection;
	using hxa7241::graphics::Vector3f;


class SpacetimeIndex
{
/// standard object services -----------------------------------------------------------------------
public:
	explicit SpacetimeIndex( const Scene& );

	virtual ~SpacetimeIndex();
	         SpacetimeIndex( const SpacetimeIndex& );
	SpacetimeIndex& operator=( const SpacetimeIndex& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual void  update( const SamplingControl& );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual const Scene& getScene() const;

	virtual bool  getNextIntersection( const Ray&,
	                                   Intersection& )                                        const;
	virtual bool  isShadowed( const Ray&,
	                          const Vector3f& position,
	                          const ObjectRenderableId& )                                     const;


/// implementation ---------------------------------------------------------------------------------
protected:
	virtual void  assign( const SpacetimeIndex& );


/// fields -----------------------------------------------------------------------------------------
private:
	const Scene*  pScene_m;
	RenderingTime time_m;
};


}}




#endif//SpacetimeIndex_h
