/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "perceptuum2.rc.h"

#include "BackgroundDialog.h"   /// own header is included last


using namespace perceptuum2::platform_ui;




/// standard object services -----------------------------------------------------------------------
BackgroundDialog::BackgroundDialog
(
	HINSTANCE hInstance,
	HWND      hWndParent
)
 :	Win32Dialog( hInstance, hWndParent ),
	grayValue_m( 0 )
{
}


BackgroundDialog::~BackgroundDialog()
{
}




/// commands ---------------------------------------------------------------------------------------
int BackgroundDialog::run
(
	udword& value
)
{
	grayValue_m = value;

	const int dlgReturn = Win32Dialog::run();
	if( dlgReturn != FALSE )
	{
		value = grayValue_m;
	}

	return dlgReturn;
}


int BackgroundDialog::run()
{
	return Win32Dialog::run();
}




/// overrides --------------------------------------------------------------------------------------
int BackgroundDialog::getDialogResourceId() const
{
	return IDD_BACKGROUND;
}


bool BackgroundDialog::dialogProc
(
	HWND   hWndDlg,
	UINT   message,
	WPARAM wParam,
	LPARAM //lParam
)
{
	bool didHandleMessage = true;

	switch( message )
	{
	case WM_INITDIALOG :
		{
			/// set max length of edit
			const int EDIT_BYTE_MAX = EDIT_LENGTH * sizeof(TCHAR);
			HWND hWndEdit = ::GetDlgItem( hWndDlg, IDE_BACKGROUND_VALUE );
			::SendMessage( hWndEdit, EM_SETLIMITTEXT, (WPARAM)EDIT_BYTE_MAX, 0 );

			/// set edit with value
			::SetDlgItemInt( hWndDlg, IDE_BACKGROUND_VALUE, grayValue_m, FALSE );

			break;
		}
	case WM_COMMAND :
		{
			if( HIWORD(wParam) == EN_UPDATE )
			{
				onEnUpdateInt( hWndDlg, IDE_BACKGROUND_VALUE, MIN_VALUE, MAX_VALUE );
			}
			else
			{
				if( wParam == IDOK )
				{
					BOOL ok;
					const UINT v = ::GetDlgItemInt( hWndDlg, IDE_BACKGROUND_VALUE, &ok, FALSE );
					/// only allow legal edit values to end the dialog
					if( ok )
					{
						grayValue_m = v;
						::EndDialog( hWndDlg, TRUE );
					}
				}
				else if( wParam == IDCANCEL )
				{
					::EndDialog( hWndDlg, FALSE );
				}
				else
				{
					didHandleMessage = false;
				}
			}
			break;
		}
	default:
		didHandleMessage = false;
	}

	return didHandleMessage;
}
