/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Win32FileDialog_h
#define Win32FileDialog_h


#include "Primitives.h"
#include "stdwin.h"
#include <string>
#include <vector>




#include "perceptuum2_platform_ui.h"
namespace perceptuum2 { namespace platform_ui {
	using namespace hxa7241::primitives;
	using std::string;
	using std::vector;


class Win32FileDialog
{
/// standard object services -----------------------------------------------------------------------
public:
	         Win32FileDialog( const TCHAR dlgTitle[], const TCHAR initialDir[], const TCHAR filter[], bool createable );
	         Win32FileDialog( const TCHAR dlgTitle[], const TCHAR initialDir[], const TCHAR filter[] );
	         Win32FileDialog( const TCHAR dlgTitle[], const TCHAR initialDir[], const vector<string>& filter, bool createable );
	         Win32FileDialog( const TCHAR dlgTitle[], const TCHAR initialDir[], const vector<string>& filter );

	virtual ~Win32FileDialog();
private:
	         Win32FileDialog( const Win32FileDialog& );
	Win32FileDialog& operator=( const Win32FileDialog& );


/// commands ---------------------------------------------------------------------------------------
public:
	        bool  run( HINSTANCE, HWND hWndParent, string& filePathname, int& namePos, int& extensionPos );
			void  setExtensionPos( int pos );

/// queries ----------------------------------------------------------------------------------------
public:
	        const vector<string>& getExtensions() const;
	        HINSTANCE getInstance()               const;


/// implementation ---------------------------------------------------------------------------------
private:
	        void  construct ( const TCHAR dlgTitle[], const TCHAR initialDir[], const TCHAR filter[], bool openNotSave, bool createable );
	static  void  makeFilter( const vector<string>& names, string& filter );
	static  void  extractExtensions( const TCHAR filter[], vector<string>& extensions );


/// fields -----------------------------------------------------------------------------------------
private:
	const TCHAR*   dialogTitle_m;
	const TCHAR*   initialDirectory_m;
	const TCHAR*   filter_m;
	DWORD          flags_m;
	const TCHAR*   pDefaultExtension_m;
	LPOFNHOOKPROC  fileDialogHook_m;

	vector<string> extensions_m;
	bool           isOpenNotSave_m;

	int            extensionPos_m;

	HINSTANCE      hInstance_m;

	static const char CLASS_NAME[];
};


}}




#endif//Win32FileDialog_h
