/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Win32ImagePaneSlideable_h
#define Win32ImagePaneSlideable_h


#include "Primitives.h"
#include "stdwin.h"

#include "hxa7241_win32.h"




#include "perceptuum2_platform_ui.h"
namespace perceptuum2 { namespace platform_ui {
	using namespace hxa7241::primitives;
	using hxa7241::win32::Win32BackBuffer;


class Win32ImagePaneSlideable
{
/// standard object services -----------------------------------------------------------------------
public:
	         Win32ImagePaneSlideable( HINSTANCE hInstance, HWND hWndParent );

	        ~Win32ImagePaneSlideable();
private:
	         Win32ImagePaneSlideable( const Win32ImagePaneSlideable& );
	Win32ImagePaneSlideable& operator=( const Win32ImagePaneSlideable& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void     moveWindow( const POINT& leftTop, const POINT& widthHeight );
	        void     setImageSize( dword width, dword height );
	        void     updateImage( dword lineNumber );
	        void     updateImage();
	        void     setBackground( COLORREF );


/// queries ----------------------------------------------------------------------------------------
public:
	        void     getImage( dword*& pixels, dword& width, dword& height ) const;
	        COLORREF getBackground()                                         const;


/// implementation ---------------------------------------------------------------------------------
private:
	static  void     registerWindowClass( HINSTANCE hInstance );
	        HWND     createWindow( HINSTANCE hInstance, HWND hWndParent );


/// message handling -------------------------------------------------------------------------------
public:
	        LRESULT  windowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );
private:
	        void     onCreate( HWND hWnd ) const;
	        void     onPaint( HWND hWnd );
	        void     onLButtonDown( HWND hWnd );
	        void     onLButtonUp( HWND hWnd );
	        void     onMouseMove( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );

	        bool     isCursorInPane( HWND hWnd )         const;
	        bool     isPaneSmallerThanImage( HWND hWnd ) const;
	        void     setCursor( HWND hWnd )              const;


/// fields -----------------------------------------------------------------------------------------
private:
	HWND             hWnd_m;
	Win32BackBuffer* pBackBuffer_m;
	HBRUSH           hBrushBackground_m;
	COLORREF         brushColor_m;

	bool             imageIsGripped_m;
	HCURSOR          normalCursor_m;
	HCURSOR          handCursor_m;
	HCURSOR          handGrippedCursor_m;

	POINT            mousePos_m;
	POINT            imagePos_m;

	/// constants
	static const TCHAR WND_CLASS_NAME[];
	static const char  CLASS_NAME[];

	static const char  ERROR_MESSAGE1[];

	static const COLORREF BACKGROUND_DEFAULT;
};


}}




#endif//Win32ImagePaneSlideable_h
