/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include <cstring>

#include "ByteSerializerOut.h"   /// own header is included last


using namespace hxa7241::utility;




/// standard object services -----------------------------------------------------------------------
ByteSerializerOut::ByteSerializerOut
(
	ostream& out
)
 :	SerializerOut( out )
{
}


ByteSerializerOut::~ByteSerializerOut()
{
}




/// implementation ---------------------------------------------------------------------------------
ByteSerializerOut& ByteSerializerOut::operator<<( const void* v )
{
	return operator<<( reinterpret_cast<udword>( v ) );
}


ByteSerializerOut& ByteSerializerOut::operator<<( const bool b )
{
	return operator<<( static_cast<ubyte>( b ) );
}


ByteSerializerOut& ByteSerializerOut::operator<<( const hxa7241::utility::byte b )
{
	return operator<<( static_cast<ubyte>( b ) );
}


ByteSerializerOut& ByteSerializerOut::operator<<( const ubyte b )
{
	out_m.put( static_cast<char>( b ) );

	return *this;
}


ByteSerializerOut& ByteSerializerOut::operator<<( const word w )
{
	return operator<<( static_cast<uword>( w ) );
}


ByteSerializerOut& ByteSerializerOut::operator<<( const uword w )
{
	out_m.put( static_cast<char>( udword(w >> 8) & udword(0xFF) ) );
	out_m.put( static_cast<char>( udword(w     ) & udword(0xFF) ) );

	return *this;
}


ByteSerializerOut& ByteSerializerOut::operator<<( const dword d )
{
	return operator<<( static_cast<udword>( d ) );
}


ByteSerializerOut& ByteSerializerOut::operator<<( const udword d )
{
	out_m.put( static_cast<char>( (d >> 24) & udword(0xFF) ) );
	out_m.put( static_cast<char>( (d >> 16) & udword(0xFF) ) );
	out_m.put( static_cast<char>( (d >>  8) & udword(0xFF) ) );
	out_m.put( static_cast<char>( (d      ) & udword(0xFF) ) );

	return *this;
}


ByteSerializerOut& ByteSerializerOut::operator<<( const long l )
{
	return operator<<( static_cast<udword>( l ) );
}


ByteSerializerOut& ByteSerializerOut::operator<<( const unsigned long l )
{
	return operator<<( static_cast<udword>( l ) );
}


ByteSerializerOut& ByteSerializerOut::operator<<( const float f )
{
	return operator<<( *(reinterpret_cast<udword*>( &f )) );
}


ByteSerializerOut& ByteSerializerOut::operator<<( const double d )
{
	float f = float(d);
	return operator<<( *(reinterpret_cast<udword*>( &f )) );
}


ByteSerializerOut& ByteSerializerOut::operator<<( const char c )
{
	out_m.put( c );

	return *this;
}


ByteSerializerOut& ByteSerializerOut::operator<<( const char* cp )
{
	if( cp != 0 )
	{
		operator<<( udword( std::strlen( cp ) ) );
		out_m << cp;
	}

	return *this;
}


ByteSerializerOut& ByteSerializerOut::operator<<( const string& s )
{
	operator<<( udword( s.length() ) );
	out_m << s.c_str();

	return *this;
}




/// implementation ---------------------------------------------------------------------------------
void ByteSerializerOut::writeHeader
(
	const char* ,//name,
	const char* typeName,
	const char* comment
)
{
	writeHeaderLine( HEADER() );
	writeHeaderLine( typeName );
	writeHeaderLine( comment );

	/// move to start of next dword
	const udword pos = out_m.tellp();
	const char zeros[] = "\0\0\0";
	out_m.write( zeros, (4 - (pos & 0x03)) & 0x03 );
}


//void ByteSerializerOut::writeFooter
//(
//	const char* ,//name,
//	const char* ,//typeName,
//	const char* //comment
//)
//{
//}
//
//
//void ByteSerializerOut::writePreObject
//(
//	const char* name,
//	const char* ,//typeName,
//	const char* //comment
//)
//{
//	out_m << name;
//}
//
//
//void ByteSerializerOut::writePostObject
//(
//	const char* ,//name,
//	const char* ,//typeName,
//	const char* //comment
//)
//{
//	out_m << "\n";
//}


void ByteSerializerOut::writeHeaderLine( const char* s )
{
	char c;
	while( (c = *(s++)) != '\0' )
	{
		if( c == '\n' )
		{
			c = ' ';
		}
		out_m.put( c );
	}
	out_m.put( '\n' );
}


const char* ByteSerializerOut::CLASS_NAME()
{
	static const char CLASS_NAME[] = "hxa7241::utility::ByteSerializerOut";
	return CLASS_NAME;
}


const char* ByteSerializerOut::HEADER()
{
	static const char HEADER[] = "hxa7241:utility:ByteSerializer version1";
	return HEADER;
}
