/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "FpToInt.h"




namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;


dword fpToIntTrunc( const float f )
{
//	static const dword SIGN_BIT         =  dword(0x80000000);   // 10000000,00000000,00000000,00000000
	static const dword EXPONENT_BITS    =  dword(0x7F800000);   // 01111111,10000000,00000000,00000000
	static const dword MANTISSA_BITS    =  dword(0x007FFFFF);   // 00000000,01111111,11111111,11111111
	static const dword IMPLICIT_1       =  dword(0x00800000);   // 00000000,10000000,00000000,00000000
	static const dword EXP_2POW_PLUS30  =  dword(0x0000009D);   // 00000000,00000000,00000000,10011101
//	static const dword EXP_2POW_PLUS31  =  dword(0x0000009E);   // 00000000,00000000,00000000,10011110
//	static const dword EXP_2POW_PLUS23  =  dword(0x00000096);   // 00000000,00000000,00000000,10010110
//	static const dword EXP_2POW_ZERO    =  dword(0x0000007F);   // 00000000,00000000,00000000,01111111
//	static const dword ONE_BITS         =  dword(0x3F800000);   // 00111111,10000000,00000000,00000000


	const dword floatBits = *(reinterpret_cast<const dword*>( &f ));   /// do NOT use a union - weird miscalculations happen !

	dword d = 0;

	if( (floatBits << 1) != 0 )
	{
		/// extract mantissa
		d = ((floatBits & MANTISSA_BITS) | IMPLICIT_1) << 7;
		const dword shift = EXP_2POW_PLUS30 - ((floatBits & EXPONENT_BITS) >> 23);
		d >>= shift;

		/// clamp to min, then max
		d &= (shift - 32) >> 31;
		d = (d | (shift >> 31)) & 0x7FFFFFFF;   /// min clamp val is DWORD_MIN+1, not DWORD_MIN

		/// set sign
		d = (d - dword(udword(floatBits) >> 31)) ^ (floatBits >> 31);
	}

	return d;


//	const dword floatBits = *(reinterpret_cast<const dword*>( &f ));   /// do NOT use a union - weird miscalculations happen !
//
//	dword d;
//
//	if( (floatBits << 1) != 0 )
//	{
//		d = 0;
//	}
//	else
//	{
//		const dword exponent = (floatBits & EXPONENT_BITS) >> 23;
//
//		/// clamp into dword range
//		if( exponent < EXP_2POW_ZERO )
//		{
//			d = 0;
//		}
//		else if( exponent > EXP_2POW_PLUS30 )
//		{
//			d = dword(0x7FFFFFFF);
//		}
//		else
//		{
//			/// extract mantissa
//			d = ((floatBits & MANTISSA_BITS) | IMPLICIT_1) << 7;
//			d >>= EXP_2POW_PLUS30 - exponent;
//		}
//
//		/// set sign
//		d = ((d - (dword)((udword)floatBits >> 31)) ^ (floatBits >> 31));
//	}
//
//	return  d;
}


dword fpToIntRound( const float f )
{
	return fpToInt( f );

//	static const dword EXPONENT_BITS    =  dword(0x7F800000);   // 01111111,10000000,00000000,00000000
//	static const dword MANTISSA_BITS    =  dword(0x007FFFFF);   // 00000000,01111111,11111111,11111111
//	static const dword IMPLICIT_1       =  dword(0x00800000);   // 00000000,10000000,00000000,00000000
//	static const dword EXP_2POW_PLUS30  =  dword(0x0000009D);   // 00000000,00000000,00000000,10011101
//
//
//	const dword floatBits = *(reinterpret_cast<const dword*>( &f ));   /// do NOT use a union
//
//	dword d = 0;
//
//	if( (floatBits << 1) != 0 )
//	{
//		/// extract mantissa
//		d = ((floatBits & MANTISSA_BITS) | IMPLICIT_1) << 8;
//
//		const int shift = EXP_2POW_PLUS30 - ((floatBits & EXPONENT_BITS) >> 23);
//
//		/// add 0.5 and shift down
//		d += 1 << shift;
//		d = (dword)( (udword)d >> (shift + 1) );
//
//		not quite correct yet...
//
//		/// clamp, to min, then max
//		d &= (shift - 32) >> 31;
//		d = (d | (shift >> 31)) & 0x7FFFFFFF;   /// min clamp val is DWORD_MIN+1, not DWORD_MIN
//
//		/// set sign
//		d = (d - dword(udword(floatBits) >> 31)) ^ (floatBits >> 31);
//	}
//
//	return d;
}




const float FRACTIONALIZER::_4BIT  = 1118481.0f / 16777216.0f;
const float FRACTIONALIZER::_8BIT  =   65793.0f / 16777216.0f;
const float FRACTIONALIZER::_16BIT =       1.0f / 16777216.0f;


}}
