/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Histogram_h
#define Histogram_h


#include "Primitives.h"
#include <vector>




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using hxa7241::utility::Float01;
	using std::vector;


/**
 * a histogram for stochastic element choosing.<br/><br/>
 *
 * a collection of float elements, each representing a weighting, is built. then,
 * by giving a uniform random, selection of an item can be made with probability
 * in proportion to the items weight.<br/><br/>
 *
 * numerically forms a probability density function from a distribution function,
 * by integrating constant slices, and searching for xs from ys.
 *
 * @implementation
 * made with std::vector and std::upper_bound.
 */
class Histogram
{
/// standard object services -----------------------------------------------------------------------
public:
	         Histogram( vector<float>& storage, dword length =0 );

	        ~Histogram();
private:
	         Histogram( const Histogram& );
	Histogram& operator=( const Histogram& );


/// commands ---------------------------------------------------------------------------------------
public:
	         void  addElement( float );


/// queries ----------------------------------------------------------------------------------------
	         bool  findElement( Float01 point, dword& index )                                 const;
	         bool  findElement( Float01 point, dword& index, float& subElementPosition )      const;

	         dword   getSize()                                                                const;
	         Float01 getElement( dword index )                                                const;


/// fields -----------------------------------------------------------------------------------------
private:
	vector<float>& composition_m;
};


}}




#endif//Histogram_h
