/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Win32ThreadLock.h"   /// implementation dependency
#include "stdwin.h"
#include <sstream>

#include "LogThreadSafe.h"


using namespace hxa7241::utility;




/// statics
const char LogThreadSafe::FILE_NAME_DEFAULT[] = "LogThreadSafe.Log";
//string     LogThreadSafe::filePathname_s( FILE_NAME_DEFAULT );   /// oops! causes a runtime crash, merely by linking this file! mad!
string     LogThreadSafe::filePathname_s;
bool       LogThreadSafe::enabled_s( false );

auto_ptr<ThreadLock> LogThreadSafe::pThreadLock_s( new hxa7241::win32::Win32ThreadLock() );   /// implementation dependency




/// standard object services -----------------------------------------------------------------------
LogThreadSafe::LogThreadSafe
(
	const char* filePathname,
	const bool  enable
)
 :	filePathname_m( filePathname ),
	enabled_m(),
	logFile_m()
{
	LogThreadSafe::enable_( enable );
}


LogThreadSafe::~LogThreadSafe()
{
}




/// commands ---------------------------------------------------------------------------------------
void LogThreadSafe::init
(
	const char*const name,
	const bool       enable
)
{
	ThreadSync sync( pThreadLock_s.get() );

	if( name != 0 )
	{
		filePathname_s = name;
		if( filePathname_s.length() == 0 )
		{
			filePathname_s = FILE_NAME_DEFAULT;
		}
	}

	enabled_s = enable;
}


void LogThreadSafe::enable( const bool b )
{
	ThreadSync sync( pThreadLock_s.get() );

	LogThreadSafe::get().enable_( b );
}


void LogThreadSafe::write( const string& s )
{
	ThreadSync sync( pThreadLock_s.get() );

	LogThreadSafe::get().write_( s );
}




/// queries ----------------------------------------------------------------------------------------
bool LogThreadSafe::isEnabled()
{
	ThreadSync sync( pThreadLock_s.get() );

	return LogThreadSafe::get().isEnabled_();
}




// implementation ----------------------------------------------------------------------------------
LogThreadSafe& LogThreadSafe::get()
{
	static LogThreadSafe instance( (filePathname_s.length() > 0) ? filePathname_s.c_str() : FILE_NAME_DEFAULT,
	                               enabled_s );

	return instance;
}


void LogThreadSafe::enable_( const bool enable )
{
	if( enable && !logFile_m.is_open() )
	{
		logFile_m.open( filePathname_m.c_str() );
	}
	enabled_m = enable;
}


void LogThreadSafe::write_( const string& s )
{
	if( enabled_m )
	{
		if( s.length() > 0 )
		{
			/// write thread id, as fixed width hex
			//{
			//	char buf[16];
			//	std::sprintf( buf, "%08X", udword(::GetCurrentThreadId()) );  //int sprintf( char* buffer, const char* format, ... );
			//	logFile_m << "\n~" << buf << "~  ";
			//}
			logFile_m << "\n~" << (void*)(::GetCurrentThreadId()) << "~  ";

			/// write string
			logFile_m << s.c_str();

			logFile_m.flush();
		}
	}
}
