/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Logger.h"   /// own header is included last


using namespace hxa7241::utility;




/// statics
const char Logger::EXC1[] = "---  ";
const char Logger::EXC2[] = "***  ";
const char Logger::BRK[]  = "\n\n";
const char Logger::DIV[]  = "\n\n----------------\n";




/// standard object services -----------------------------------------------------------------------
Logger::Logger()
 :	buffer_m()
{
	Logger::init( 6, false );
}


Logger::Logger
(
	const dword fpPrecision,
	const bool  isFpScientific
)
 :	buffer_m()
{
	Logger::init( fpPrecision, isFpScientific );
}


Logger::Logger
(
	const bool  isFpScientific,
	const dword fpPrecision
)
 :	buffer_m()
{
	Logger::init( fpPrecision, isFpScientific );
}


Logger::Logger
(
	const dword fpPrecision
)
 :	buffer_m()
{
	Logger::init( fpPrecision, false );
}


Logger::Logger
(
	const bool  isFpScientific
)
 :	buffer_m()
{
	Logger::init( 6, isFpScientific );
}


Logger::~Logger()
{
	flush();
}




/// queries ----------------------------------------------------------------------------------------
//const char* Logger::EXC1()
//{
//	static const char k[] = "---  ";
//	return k;
//}
//
//
//const char* Logger::EXC2()
//{
//	static const char k[] = "***  ";
//	return k;
//}
//
//
//const char* Logger::BRK()
//{
//	static const char k[] = "\n\n";
//	return k;
//}
//
//
//const char* Logger::DIV()
//{
//	static const char k[] = "\n\n----------------\n";
//	return k;
//}




/// implementation ---------------------------------------------------------------------------------
void Logger::init
(
	const dword fpPrecision,
	const bool  isFpScientific
)
{
	buffer_m.setf( buffer_m.boolalpha );

	buffer_m.setf( buffer_m.showpoint );
	//buffer_m.setf( buffer_m.showpos );
	buffer_m.setf( isFpScientific ? buffer_m.scientific : buffer_m.fixed, buffer_m.floatfield );
	//buffer_m.width( fpPrecision > 0 ? fpPrecision : 1 );
	buffer_m.precision( fpPrecision > 0 ? fpPrecision : 1 );
}


void Logger::flush()
{
	LogThreadSafe::write( buffer_m.str() );

	buffer_m.str( string() );
}
