/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef RandomGood_h
#define RandomGood_h


#include "Primitives.h"
#include "Random.h"




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;


/**
 * good random number generator.<br/><br/>
 *
 * gives a signed dword in the range [1, MAX_VALUE], and a float in the
 * range (0, 1).<br/><br/>
 *
 * adapted from: <cite>'Numerical Recipes In C', ch7 p280-281, www.nr.com.</cite>
 */
class RandomGood
	: public Random
{
public:
/// standard object services -----------------------------------------------------------------------
	explicit RandomGood( dword seed =0 );

	virtual ~RandomGood();
	         RandomGood( const RandomGood& );
	RandomGood& operator=( const RandomGood& );


/// commands ---------------------------------------------------------------------------------------
	virtual void  setSeed( dword );
	virtual void  next();


/// queries ----------------------------------------------------------------------------------------
	virtual dword getDword() const;
	virtual float getFloat() const;


/// serialization ----------------------------------------------------------------------------------
public:
	friend  SerializerOut& operator<<( SerializerOut&, const RandomGood& );
	friend  SerializerIn&  operator>>( SerializerIn&,        RandomGood& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// fields -----------------------------------------------------------------------------------------
private:
	/// constants
	static const dword IA   = 16807;
	static const dword IM   = 2147483647;
	static const dword IQ   = 127773;
	static const dword IR   = 2836;
	static const dword NTAB = 32;
	static const dword NDIV = 1 + (IM-1) / NTAB;
	static const float AM;

	dword idum_m;
	dword iy_m;
	dword iv_m[ NTAB ];
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const RandomGood& );
SerializerIn&  operator>>( SerializerIn&,        RandomGood& );


}}




#endif//RandomGood_h
