/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "SerializerIn.h"   /// own header is included last


using namespace hxa7241::utility;




/// statics
const char SerializerIn::CLASS_NAME[]        = "hxa7241::utility::SerializerIn";
const char SerializerIn::EXCEPTION_MESSAGE[] = "serialization stream failure";




/// standard object services -----------------------------------------------------------------------
SerializerIn::SerializerIn
(
	istream& in,
	const char* fileName
)
 :	in_m( in ),
	buffer_m( 128, '\0' ),
	fileName_m( fileName ),
	previousExceptionFlags_m()
{
	const char METHOD_NAME[] = "constructor";

	try
	{
		previousExceptionFlags_m = in_m.exceptions();
		in_m.exceptions( istream::badbit | istream::failbit | istream::eofbit );
	}
	catch( std::ios_base::failure e )
	{
		throw hxa7241::utility::Exception( EXCEPTION_MESSAGE, CLASS_NAME, METHOD_NAME );
	}
}


SerializerIn::~SerializerIn()
{
	in_m.exceptions( previousExceptionFlags_m );
}




/// implementation ---------------------------------------------------------------------------------
SerializerIn& SerializerIn::operator>>( void*& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( bool& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( hxa7241::utility::byte& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( ubyte& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( word& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( uword& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( dword& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( udword& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( long& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( unsigned long& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( float& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( double& a )
{
	in_m >> a;

	return *this;
}


SerializerIn& SerializerIn::operator>>( char& c )
{
	in_m >> c;

	return *this;
}


SerializerIn& SerializerIn::operator>>( char* cp )
{
	/// get first non-blank char
	char c;
	in_m >> c;

	/// get chars until source or target contains a 0
	for( ;  (c != '\0') && (*cp != '\0');  c = char(in_m.get()) )
	{
		*(cp++) = c;
	}
	*cp = '\0';

	return *this;
}


SerializerIn& SerializerIn::operator>>( string& s )
{
	/// get first non-blank char
	char c;
	in_m >> c;

	/// get chars until source contains a 0
	s = "";
	for( ;  c != '\0';  c = char(in_m.get()) )
	{
		s += c;
	}

	return *this;
}




/// implementation ---------------------------------------------------------------------------------
void SerializerIn::readHeader
(
	const char* //name
)
{
	/// default do nothing
}


void SerializerIn::readFooter
(
	const char* //name
)
{
	/// default do nothing
}


bool SerializerIn::readPreObject
(
	const char* //name
)
{
	/// default do nothing
	return true;
}


void SerializerIn::readPostObject
(
	const char* //name
)
{
	/// default do nothing
}


bool SerializerIn::readPreElement
(
	const char* ,//collectionName,
	const char* ,//name,
	bool&       isCollectionEnd
)
{
	/// default do nothing much
	isCollectionEnd = true;
	return true;
}
