/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef TextSerializerOut_h
#define TextSerializerOut_h


#include "Primitives.h"
#include "XmlSerializerOut.h"
#include <fstream>
#include <string>




#include "utility.h"
namespace hxa7241 { namespace utility {
	using namespace hxa7241::primitives;
	using std::ostream;
	using std::string;


/**
 * a basic unformatted text file outward serializer.<br/><br/>
 *
 * writes each sub object on a separate line, indented according to containment
 * depth.
 */
class TextSerializerOut
	: public XmlSerializerOut
{
/// standard object services -----------------------------------------------------------------------
protected:
	explicit TextSerializerOut( ostream& );

	virtual ~TextSerializerOut();
private:
	         TextSerializerOut( const TextSerializerOut& );
	TextSerializerOut& operator=( const TextSerializerOut& );


/// commands ---------------------------------------------------------------------------------------
public:
	template<class TYPE>
	static  void  serializeOut( const TYPE& object, const char* filePathname );
	template<class TYPE>
	static  void  serializeOut( const TYPE& object, ostream& );


/// queries ----------------------------------------------------------------------------------------
public:


/// implementation ---------------------------------------------------------------------------------
protected:
	/// overrides
	virtual void  writeHeader( const char* name, const char* typeName, const char* comment );
	virtual void  writeFooter( const char* name, const char* typeName, const char* comment );

	virtual void  writePreObject ( const char* name, const char* typeName, const char* comment );
	virtual void  writePostObject( const char* name, const char* typeName, const char* comment );

	virtual void  writeString  ( const char* ) const;


/// fields -----------------------------------------------------------------------------------------
private:
	static const char CLASS_NAME[];
};








/// TEMPLATES ///


/// commands ---------------------------------------------------------------------------------------
/// static
template<class TYPE>
void TextSerializerOut::serializeOut
(
	const TYPE&      object,
	const char*const filePathname
)
{
	std::ofstream file( filePathname, std::ofstream::trunc );

	TextSerializerOut::serializeOut( object, file );
}


template<class TYPE>
void TextSerializerOut::serializeOut
(
	const TYPE& object,
	ostream&    out
)
{
	TextSerializerOut tso( out );

	tso.write( object );
}


}}




#endif//TextSerializerOut_h
