/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "Exception.h"
#include "Logger.h"
#include <process>

#include "Thread.h"


using namespace hxa7241::utility;




/// standard object services -----------------------------------------------------------------------
Thread::Thread( Runnable*const r )
 :	runnable_m( r )
{
	if( runnable_m == 0 )
	{
		throw hxa7241::utility::Exception( "null runnable given", CLASS_NAME(), "constructor" );
	}
}


Thread::~Thread()
{
}




/// commands ---------------------------------------------------------------------------------------
void Thread::stopForcibly()
{
}


void Thread::setPriority( const dword )
{
}


void Thread::suspend()
{
}


void Thread::resume()
{
}




/// queries ----------------------------------------------------------------------------------------
dword Thread::getPriorityRange()  const
{
	return 0;
}


const char* Thread::CLASS_NAME()
{
	static const char CLASS_NAME[] = "hxa7241::utility::Thread";
	return CLASS_NAME;
}




/// implementation ---------------------------------------------------------------------------------
unsigned __stdcall hxa7241::utility::threadFunction( void* pVoid )
{
	using hxa7241::utility::Logger;
	Logger() << "< thread started";

	unsigned returnVal = 0;
	try
	{
		Runnable* runnable = static_cast<Runnable*>( pVoid );
		if( runnable == 0 )
		{
			throw hxa7241::utility::Exception( "null runnable given", Thread::CLASS_NAME(), "threadFunction" );
		}

		runnable->run();

		returnVal = 1;
	}
	catch( const std::exception& e )
	{
		Logger() << Logger::EXC2 << e.what();
	}
	catch( ... )
	{
		hxa7241::utility::Exception e;
		Logger() << Logger::EXC2 << e.what();
	}

	Logger() << "> thread about to stop";
	return returnVal;//std::_endthreadex( returnVal );
}
