/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef hxa7241vector_h
#define hxa7241vector_h


#include <vector>




#include "hxa7241_utility.h"
namespace hxa7241 { namespace utility {


/**
 * deallocates a std::vector reserve.
 */
template<class TYPE>
void rezero( std::vector<TYPE>& v )
{
	if( v.capacity() != 0 )
	{
		std::vector<TYPE> tmp;
		v.swap( tmp );
	}
}


/**
 * makes a std::vector reserve allocation equal its size.
 */
template<class TYPE>
void unreserve( std::vector<TYPE>& v )
{
	if( v.capacity() != v.size() )
	{
		std::vector<TYPE> tmp( v );
		v.swap( tmp );
	}
}


/**
 * sets a std::vector size and reserve allocation to the same specified amount.
 */
template<class TYPE>
void resizeReserve( std::vector<TYPE>& v, int size )
{
	if( size < 0 )
	{
		size = 0;
	}

	if( v.capacity() != unsigned(size) )
	{
		std::vector<TYPE> tmp;
		tmp.reserve( size );

		if( v.size() <= unsigned(size) )
		{
			tmp = v;
			tmp.resize( size );
		}
		else
		{
			v.resize( size );
			tmp = v;
		}

		v.swap( tmp );
	}
	else
	{
		v.resize( size );
	}
}


/**
 * makes a std::vector empty with the specified reserve allocation.
 */
template<class TYPE>
void reserveExactClear( std::vector<TYPE>& v, int size )
{
	if( size < 0 )
	{
		size = 0;
	}

	if( v.capacity() != unsigned(size) )
	{
		std::vector<TYPE> tmp;
		tmp.reserve( size );
		v.swap( tmp );
	}
	else
	{
		v.clear();
	}
}


/**
 * assigns to a std::vector and makes the reserve allocation equal its size.
 */
template<class TYPE>
void assignUnreserve( const std::vector<TYPE>& other, std::vector<TYPE>& v )
{
	if( v.capacity() != other.size() )
	{
		std::vector<TYPE> tmp( other );
		v.swap( tmp );
	}
	else
	{
		v = other;
	}
}


}}




#endif//hxa7241vector_h
