/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Win32BackBuffer_h
#define Win32BackBuffer_h


#include "Primitives.h"
#include "stdwin.h"




#include "hxa7241_win32.h"
namespace hxa7241 { namespace win32 {
	using namespace hxa7241::primitives;
	using hxa7241::win32::Win32Dib;


class Win32BackBuffer
{
/// standard object services -----------------------------------------------------------------------
public:
	                  Win32BackBuffer( HDC frontDc, dword width, dword height, const dword palette[] );
	                  Win32BackBuffer( HDC frontDc, dword width, dword height, dword bitsPerPixel, const dword palette[] );
	                  Win32BackBuffer( HDC frontDc, dword width, dword height );

	virtual          ~Win32BackBuffer();
private:
	                  Win32BackBuffer( const Win32BackBuffer& );
	Win32BackBuffer&  operator=( const Win32BackBuffer& );


/// construction -----------------------------------------------------------------------------------
private:
	        void construct( HDC frontDc, dword width, dword height, dword bitsPerPixel, const dword palette[] );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual HDC       getDC();
	virtual Win32Dib* getWin32Dib();


/// queries ----------------------------------------------------------------------------------------
public:
	virtual dword     getWidth()                                         const;
	virtual dword     getHeight()                                        const;

	virtual void      blitToCenter ( HDC dc, dword width, dword height ) const;
	virtual void      blitToStretch( HDC dc, dword width, dword height ) const;
	virtual void      blitToScale  ( HDC dc, dword width, dword height ) const;

	virtual void      blitToPositionWithBorder( HDC dc, const POINT& targetSize,
	                                            const RECT* pSourcePortion, POINT& position,
	                                            HBRUSH border )                              const;


/// implementation ---------------------------------------------------------------------------------
	static  void      calcCentering( const POINT& targetWidHei, const POINT& sourceWidHei,
	                                 POINT& widHei, POINT& offsetTarget, POINT& offsetSource );


/// fields -----------------------------------------------------------------------------------------
private:
	        HDC       dc_m;
	        HGDIOBJ   defaultBitmap_m;
	        Win32Dib* pWin32Dib_m;

	static const char CLASS_NAME[];
	static const char EXCEPTION_MESSAGE1[];
	static const char EXCEPTION_MESSAGE2[];
	static const char EXCEPTION_MESSAGE3[];
};


}}




#endif//Win32BackBuffer_h
