/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include <sstream>

#include "Win32Exception.h"


/// for borland c++ 5.5.1 (another at the end of the file)
#ifdef BORLANDCPP551
#pragma option push -b -a8 -pc -Vx- -Ve- -w-inl -w-aus -w-sig
#endif


using namespace hxa7241::win32;




/// externs
extern HINSTANCE hInstance_g;


/// statics
const char Win32Exception::DEFAULT_WIN32MESSAGE[] = "unknown win32 error";




/// standard object services -----------------------------------------------------------------------
Win32Exception::Win32Exception
(
	const UINT  messageResourceId,
	const char* pClassName,
	const char* pMethodName
) throw()
 :	Exception( 0, pClassName, pMethodName ),
	messageResourceId_m( messageResourceId ),
	errorCode_m( 0 )
{
}


Win32Exception::Win32Exception
(
	const UINT  messageResourceId,
	const DWORD errorCode,
	const char* pClassName,
	const char* pMethodName
) throw()
 :	Exception( 0, pClassName, pMethodName ),
	messageResourceId_m( messageResourceId ),
	errorCode_m( errorCode )
{
}


Win32Exception::Win32Exception
(
	const char* pMessage,
	const DWORD errorCode,
	const char* pClassName,
	const char* pMethodName
) throw()
 :	Exception( pMessage, pClassName, pMethodName ),
	messageResourceId_m( 0 ),
	errorCode_m( errorCode )
{
}


Win32Exception::~Win32Exception() throw()
{
}


Win32Exception::Win32Exception( const Win32Exception& other ) throw()
 :	Exception( other ),
	messageResourceId_m( other.messageResourceId_m ),
	errorCode_m( other.errorCode_m )
{
}


Win32Exception& Win32Exception::operator=( const Win32Exception& other ) throw()
{
	if( &other != this )
	{
		Exception::operator=( other );

		messageResourceId_m = other.messageResourceId_m;
		errorCode_m         = other.errorCode_m;
	}

	return *this;
}




/// implementation ---------------------------------------------------------------------------------
string& Win32Exception::appendMessage( string& s ) const
{
	/// add resource string message
	bool ok = false;
	if( messageResourceId_m != 0 )
	{
		TCHAR buf[128];
		ok = bool( ::LoadString( hInstance_g, messageResourceId_m, (LPSTR)buf, sizeof(buf)/sizeof(TCHAR) ) );
		if( ok )
		{
			append( buf, s );
		}
	}
	if( !ok )
	{
		Exception::appendMessage( s );
	}


	/// add win32 error message
	if( errorCode_m != 0 )
	{
		s += "\n\t";
		appendWin32Message( s );
		s += "  (win32 code: ";
		{
			std::ostringstream ss;
			ss << errorCode_m;
			s += ss.str();
		}
		s += ")";
	}

	return s;
}


string& Win32Exception::appendWin32Message( string& s ) const
{
	TCHAR buf[128];
	const bool ok = bool( ::FormatMessage( FORMAT_MESSAGE_FROM_SYSTEM, 0, errorCode_m, 0, (LPTSTR)buf, sizeof(buf)/sizeof(TCHAR), 0 ) );
	if( ok )
	{
		append( buf, s );

		/// FormatMessage idiotically adds newlines, so now they have to be removed
		/// thank athena for the c++ library at least...
		s.erase( s.length() - 2 );
	}
	else
	{
		s += DEFAULT_WIN32MESSAGE;
	}

	return s;
}


string& Win32Exception::append( const TCHAR* tcs, string& s )
{
	while( *tcs != TCHAR(0) )
	{
		s += char(*tcs);
		++tcs;
	}

	return s;
}


void Win32Exception::assign
(
	const Exception& other
)
{
	if( &other != this )
	{
		Exception::operator=( other );

		messageResourceId_m = 0;
		errorCode_m         = 0;
	}
}




#ifdef BORLANDCPP551
#pragma option pop
#endif
